#include "StdAfx.h"
#include "ConfigFile.h"

char *g_PortType[] = {PORTSTR_COM,PORTSTR_USB};

CConfigFile::CConfigFile()
{
	m_pFilePointer = NULL;
	memset(&m_ShutdownParam, 0, sizeof(m_ShutdownParam));
}

CConfigFile::~CConfigFile()
{
	
}

int CConfigFile::OpenFile(char *pszFileName, char *pszMode)
{
	if (NULL != m_pFilePointer)
	{
		fclose(m_pFilePointer);
		m_pFilePointer = NULL;
	}
	if((fopen_s(& m_pFilePointer, pszFileName, pszMode)) == 0)
	{
		return 1;
	}
	
 	return 0;
}

int CConfigFile::CloseFile()
{
	if(m_pFilePointer != NULL)
	{
		fclose(m_pFilePointer);
		m_pFilePointer = NULL;
		return 1;
	}
	
	return 0;
}

int CConfigFile::FindSection(char *pszSection)
{
	size_t 	iLen = 0;
	char 	szBuf[MAX_LINELENGTH];
	memset(szBuf, 0, sizeof(szBuf));
	
	if((pszSection == NULL) || (m_pFilePointer == NULL))
	{
		return 0;
	}
	
	fseek(m_pFilePointer, 0L, SEEK_SET);
	
	while(fgets(szBuf, MAX_LINELENGTH, m_pFilePointer))
	{
		iLen = strlen(szBuf);
		if(iLen > 1)
		{
			szBuf[iLen-1] = 0;
		}
		if(strcmp(szBuf, pszSection) == 0)
		{
			return 1;
		}
		memset(szBuf, 0, sizeof(szBuf));
	}
	
	return 0;
}

int CConfigFile::GetString(char *pszItem, char *pszItemVal)
{
	char 	szBuf[MAX_LINELENGTH];
	char	szCurItem[MAX_LINELENGTH];

	do
	{
		memset(szBuf, 0, sizeof(szBuf));
		memset(szCurItem, 0, sizeof(szCurItem));
		if(!fgets(szBuf, MAX_LINELENGTH, m_pFilePointer))  
		{
			return -1;                                              
		}
		if(!GetItemFromLine(szBuf, szCurItem, pszItemVal))
		{
			return -1;
		}
	} while(strcmp(pszItem, szCurItem));

	return 1;
}

int CConfigFile::GetInt(char *pszItem)
{
	char	szBuf[MAX_LINELENGTH];
	char 	szCurItem[MAX_LINELENGTH];
	char	szCurItemValue[MAX_LINELENGTH];
	
	do
	{
		memset(szBuf, 0, sizeof(szBuf));
		memset(szCurItem, 0, sizeof(szCurItem));
		memset(szCurItemValue, 0, sizeof(szCurItemValue));
	
		if(!fgets(szBuf, MAX_LINELENGTH, m_pFilePointer))
		{
			return -1;
		}
		if(!GetItemFromLine(szBuf, szCurItem, szCurItemValue))
		{
			return -1;
		}
	} while(strcmp(pszItem, szCurItem));
	
	return	atoi(szCurItemValue);
}

int CConfigFile::GetItemFromLine(char *pszBuf, char *pszItem, char *pszItemValue)
{
	UINT	nStartPos = 0;
	char 	szTemp[MAX_LINELENGTH];
	
	memset(szTemp, 0, sizeof(szTemp));
	
	GetStringFromLine(pszItem, pszBuf, &nStartPos, '=');
	
	if (pszItem == NULL)
	{
		return 1;
	}
	
	if(pszItem[0] == '[')
	{
		return 0;
	}
	
	if(nStartPos == ENDOFLINE)
	{
		return 1;
	}
		
	GetStringFromLine(szTemp, pszBuf, &nStartPos, '\n');
	
	if(strlen(szTemp) >= 256)
	{
		return 0;
	}
	
	strcpy_s(pszItemValue, MAX_LINELENGTH, szTemp);
	
	return 1;
}

int CConfigFile::GetStringFromLine(char *pszDest, char *pszBuf, UINT *nStartPos, char szSeparator)
{
	char 	szCur;
	UINT 	nPos;
	char	szTemp[MAX_LINELENGTH];
	int		i = 0;
	
	memset(szTemp, 0, sizeof(szTemp));
	nPos = *nStartPos;
	
	if(nPos == ENDOFLINE)
	{
		return 0;
	}
	
	szCur = pszBuf[nPos];
	
	while((szCur != szSeparator) && (szCur != '\0')) 
	{
		szTemp[i++] = szCur;
		nPos++;
		szCur = pszBuf[nPos];
	}	
	
	if (szCur == '\n')
	{
		nPos = ENDOFLINE;
	}
	else
	{
		nPos++;
	}
	
	memset(pszDest, 0, MAX_LINELENGTH);
	TrimString(pszDest, szTemp);
	*nStartPos = nPos;

	return 1;
}

int CConfigFile::TrimString(char *pszDest, char *pszStr)
{
	size_t 	iStartPos = 0, iEndPos = 0;
	size_t 	iLength = 0;
	char 	*ptr = NULL;
		
	iLength = strlen(pszStr);
	iEndPos = iLength;
	
	if(iLength == 0)
	{
		pszDest[0] = '\0';
		return 0;
	}
	
	ptr = pszStr;
	while((*ptr != '\0') && (*ptr != '\n'))
	{
		if((*ptr == 32) || (*ptr == '\t'))
		{
			iStartPos++;
			ptr++;
		}
		else
		{
			break;
		}
	}
	
	ptr = pszStr;
	while((*ptr != '\0') && (*ptr != '\n'))
	{
		ptr++;
	}
	ptr--;
	
	while((*ptr != '\0') && (*ptr != '\n'))
	{
		if((*ptr == 32) || (*ptr == '\t'))
		{
			iEndPos--;
			ptr--;
		}
		else
		{
			break;
		}
	}

	if(iEndPos > iStartPos)
	{
		memcpy(pszDest, pszStr+iStartPos, iEndPos-iStartPos);
	}
	else
	{
		pszDest[0] = '\0';
		return 0;
	}
	
	return 1;
}

int CConfigFile::GetShutdownParameter(PSHUTDOWNPARAM pShutdownParam)
{
	int		iValue = 0;
	int		iNValue = 0;
	int		iFilePointer = 0;
	char	szValue[MAX_EXTERNCMD_LEN];
	char	FilePath[MAX_PATH];

	if(pShutdownParam == NULL)
	{
		return 0;
	}

	memset(pShutdownParam,0,sizeof(SHUTDOWNPARAM));

	if(!GetFilePath(FilePath, CONFIG_FILENAME))
	{
		return 0;
	}

	if((OpenFile(FilePath, "r+") == 0) || (!FindSection(ShutdownParamSection)))
	{
		SetDefaultShutdownParam(&m_ShutdownParam); 
		memcpy(pShutdownParam,&m_ShutdownParam,sizeof(SHUTDOWNPARAM));
		if(WriteConfigurationFile() == 1)
		{
			return 1;
		}
		else
		{
			return 0;
		}	
	}
	
	iFilePointer = ftell(m_pFilePointer);
	fseek(m_pFilePointer, iFilePointer, SEEK_SET);


	if((iNValue = GetInt(S_SearchType)) == -1)
	{
		pShutdownParam->uSearchType = SEARCHTYPE_DEFAULT; 
	}
	else                                                      
	{ 
		pShutdownParam->uSearchType = iNValue;
	}

	if((iValue = GetInt(S_PortType)) == -1)
	{
		pShutdownParam->uPortType = PORTTYPE_DEFAULT; 
	}
	else
	{
		pShutdownParam->uPortType = iValue;
	}

	if((iValue = GetInt(S_PortNum)) == -1)
	{
		pShutdownParam->uComPort = COMPORT_DEFAULT;
	}
	else
	{
		pShutdownParam->uComPort = iValue;
	}


	if((iValue = GetInt(S_ACFailDelayTime)) == -1)
	{
		pShutdownParam->uACFailDelayTime = ACFAILDELAY_DEFAULT;
	}
	else
	{
		pShutdownParam->uACFailDelayTime = iValue;
	}

	if((iValue = GetString(S_ExtCmd,szValue)) == -1)
	{
		strcpy_s(pShutdownParam->szExternCmd,MAX_EXTERNCMD_LEN, EXTERNCMD_DEFAULT);
	}
	else
	{
		strcpy_s(pShutdownParam->szExternCmd,MAX_EXTERNCMD_LEN,szValue);
	}

	if((iValue = GetInt(S_CommandDelayTime)) == -1)
	{
		pShutdownParam->uCommandDelayTime = EXTERNCMD_DELAY_DEFAULT;
	}
	else
	{
		pShutdownParam->uCommandDelayTime = iValue;
	}
	
	if((iValue = GetInt(S_ShutdownNeedTime)) == -1)
	{
		pShutdownParam->uShutdownNeedTime = SDNEEDTIME_DEFAULT;
	}
	else
	{
		pShutdownParam->uShutdownNeedTime = iValue;
	}

	CloseFile();

	
	return 1;
}

void CConfigFile::SetDefaultShutdownParam(PSHUTDOWNPARAM pShutdownParam)
{
	if(pShutdownParam != NULL)
	{
		pShutdownParam->uSearchType = SEARCHTYPE_DEFAULT;        //AutoSearch
		pShutdownParam->uPortType = PORTTYPE_DEFAULT;            //0
		pShutdownParam->uComPort = COMPORT_DEFAULT;              //1
		pShutdownParam->uACFailDelayTime = ACFAILDELAY_DEFAULT;
		pShutdownParam->uShutdownNeedTime = SDNEEDTIME_DEFAULT;
		pShutdownParam->uCommandDelayTime = EXTERNCMD_DELAY_DEFAULT;
		strcpy_s(pShutdownParam->szExternCmd,MAX_EXTERNCMD_LEN, EXTERNCMD_DEFAULT);
	}
}

int CConfigFile::SetShutdownParameter(PSHUTDOWNPARAM pShutdownParam)
{
	memcpy(&m_ShutdownParam, pShutdownParam, sizeof(m_ShutdownParam));
	return WriteConfigurationFile();
}
BOOL CConfigFile::GetFilePath(char * FilePath,const char * FileName)
{
	int		i = 0;
	TCHAR	szFileName[MAX_PATH] = {0};

	if(GetModuleFileName(NULL, szFileName, MAX_PATH))
	{
		for(i=(int)_tcslen(szFileName); i>=0; i--)
		{
			if(szFileName[i]== '\\')
				break;
		}

		szFileName[i+1] = 0;
		strcat_s(szFileName, FileName);

		if(FilePath != NULL)
		{
			strcpy_s(FilePath,MAX_PATH, szFileName);
		}

		return TRUE;
	}

	return FALSE;
}
int CConfigFile::WriteConfigurationFile()
{
	char 	szTemp[MAX_LINELENGTH];
	char 	szBuf[MAX_LINELENGTH];
	char	FilePath[MAX_PATH];
	
	GetFilePath(FilePath,CONFIG_FILENAME);

	if(OpenFile(FilePath, "w+") == 0)
	{
		return 0;
	}

    WriteOneLine(ShutdownParamSection, 1);								
	
	sprintf_s(szTemp, MAX_LINELENGTH, "%s = %d", S_SearchType ,m_ShutdownParam.uSearchType);
	WriteOneLine(szTemp,1);		

	sprintf_s(szTemp, MAX_LINELENGTH, "%s = %d", S_PortType ,m_ShutdownParam.uPortType);
	WriteOneLine(szTemp,1);	

	sprintf_s(szBuf, MAX_LINELENGTH, " = %d", m_ShutdownParam.uComPort);
	sprintf_s(szTemp, MAX_LINELENGTH,  "%s%s", S_PortNum, szBuf);
	WriteOneLine(szTemp, 1);
	
	sprintf_s(szBuf, MAX_LINELENGTH, " = %d", m_ShutdownParam.uACFailDelayTime);
	sprintf_s(szTemp, MAX_LINELENGTH, "%s%s", S_ACFailDelayTime, szBuf);
	WriteOneLine(szTemp, 1);

	sprintf_s(szTemp, MAX_LINELENGTH, "%s = %s", S_ExtCmd, m_ShutdownParam.szExternCmd);
	WriteOneLine(szTemp, 1);

	sprintf_s(szBuf, MAX_LINELENGTH, " = %d", m_ShutdownParam.uCommandDelayTime);
	sprintf_s(szTemp, MAX_LINELENGTH, "%s%s", S_CommandDelayTime, szBuf);
	WriteOneLine(szTemp, 1);

	sprintf_s(szBuf, MAX_LINELENGTH, " = %d", m_ShutdownParam.uShutdownNeedTime);
	sprintf_s(szTemp, MAX_LINELENGTH, "%s%s", S_ShutdownNeedTime, szBuf);
	WriteOneLine(szTemp, 2);
	
	WriteOneLine(EndSection, 1);
	
	CloseFile();

	return 1;
}

int CConfigFile::WriteOneLine(char *pszText, UINT uReturnNum)         
{
	size_t	uSize = 0;
	char	pszTemp[2] = {0};
	UINT	i = 0;
	
	if(m_pFilePointer == NULL)
	{
		return 0;
	}
	
	pszTemp[0] = '\n';
	pszTemp[1] = 0;
	
	if(pszText != NULL )
	{
		uSize = strlen(pszText);
		fwrite(pszText, sizeof(char), uSize, m_pFilePointer);
	}
	
	for(i=0; i<uReturnNum; i++)
	{
		fwrite(pszTemp, sizeof(char), 1, m_pFilePointer);
	}
	
	return 1;
}
