#include "StdAfx.h"

#include "ups.h"
#include "ListPorts.h"
#include "..\Protocol.h"
#include <STRING>


//*****************************************Data maintance area**************************
#define MAXIN_BUFF                          4096
#define MAXOUT_BUFF                         4096 
#define BITESIZE_TRANSMIT_ONCE              8
#define CTS_TRUE_OR_FALSE                   0
#define DSR_TRUE_OR_FALSE                   0
#define INTRANSMIT_TRUE_OR_FALSE            1
#define OUTTRANSMIT_TRUE_OR_FALSE           1
#define MININPUT_INBUFF                     100
#define MAXINPUT_INBUFF                     100
#define WAIT_TIME                           500
#define ZERO                                0
#define MAX_COMMAND_LENGTH					20
#define READ_DELAY							100
//*****************************************************************************************


#define MUTEX_FOR_OPERATION_UPS  "Global\\MutexForOperationUPS"

DWORD			CUPS::m_dwComPortCount = 0;
TCHAR			CUPS::m_szPort[MAX_PORT_NUM][MAX_PORTNAME_LEN] = {0};

static	TCHAR	szPort[MAX_COMPORT_NUM][MAX_PORTNAME_LEN];
static	DWORD	dwComPortNum = 0;

#define SetStatus(status, bit)      (status |= bit)
#define ClearStatus(status, bit)    (status &= (0xFFFFFFFF - bit))
#define CheckBit(status, bit)       ((status & bit) == bit)

POpenOneUSBDevice OpenOneUSBDevice = NULL;
PReadOneUSBDevice ReadOneUSBDevice = NULL;
PWriteOneUSBDevice WriteOneUSBDevice = NULL;
POpenUSBDeviceByName OpenUSBDeviceByName = NULL;

static BOOL CALLBACK GetComPort(LPVOID lpCallbackValue,LISTPORTS_PORTINFO* lpPortInfo)
{
	size_t length = wcslen((wchar_t *)(lpPortInfo->lpPortName));
	memcpy(szPort[dwComPortNum++], lpPortInfo->lpPortName, 2*length);
	return TRUE;
}

void CheckStatus(ULONG ulStatus, ULONG ulBit, DWORD &wSet, DWORD wBit)
{
	if(CheckBit(ulStatus, ulBit))
	{
		SetStatus(wSet, wBit);
	}
	else
	{
		ClearStatus(wSet, wBit);
	}
}

CUPS::CUPS(void)
{
	m_hPortHandle = NULL;
	m_hUsbDll = NULL;
}

CUPS::~CUPS(void)
{
	if (m_hUsbDll)
	{
		FreeLibrary(m_hUsbDll);
		CloseHandle(m_hUsbDll);
	}
}

BOOL CUPS::Initialize()
{
	m_PortType = USB;
	memset(&m_UsbDevice, 0, sizeof(m_UsbDevice));
	memset(&m_upsInfo,0,sizeof(m_upsInfo));
	m_dwUPSStatus = 0;

	char	FilePath[MAX_PATH];
	char	FilePath1[MAX_PATH];
	char	*pResult = NULL;
	char	ch='\\';

	memset(FilePath,0,MAX_PATH);
	memset(FilePath1,0,MAX_PATH);

	GetModuleFileName(NULL,FilePath1,MAX_PATH);
	pResult=strrchr(FilePath1,ch);

	memcpy(FilePath,FilePath1,pResult-FilePath1+1);
	strcat_s(FilePath,USBDLL_PATH);

	if ((m_hUsbDll = LoadLibrary(FilePath)) == NULL)
	{
		return FALSE;
	}
	
	if ((ReadOneUSBDevice = (PReadOneUSBDevice)GetProcAddress(m_hUsbDll,(LPCSTR)"ReadOneUSBDevice")) == NULL)
	{
		return FALSE;
	}

	if ((WriteOneUSBDevice = (PWriteOneUSBDevice)GetProcAddress(m_hUsbDll,(LPCSTR)"WriteOneUSBDevice")) == NULL)
	{
		return FALSE;
	}

	if ((OpenOneUSBDevice = (POpenOneUSBDevice)GetProcAddress(m_hUsbDll,(LPCSTR)"OpenOneUSBDevice")) == NULL)
	{
		return FALSE;
	}

	if ((OpenUSBDeviceByName = (POpenUSBDeviceByName)GetProcAddress(m_hUsbDll,(LPCSTR)"OpenUSBDeviceByName")) == NULL)
	{
		return FALSE;
	}
	
	return TRUE;
}

void CUPS::CloseConnection(PortType pt)
{
	if(m_hPortHandle != NULL)
	{
		if(pt == COM)
		{
			SetCommMask(m_hPortHandle, 0);
			EscapeCommFunction(m_hPortHandle, CLRDTR); // now this is changed to original.
			PurgeComm(m_hPortHandle, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
		}
		CloseHandle(m_hPortHandle);
		m_hPortHandle = NULL;
	}
}

//Return value:
//		Return total com port
DWORD CUPS::ListComPorts(DWORD *dwComID)
{
	// Auto detect Com Port in pc
	CListPorts	listport; 
	UINT		i = 0;
	UINT		uCount = 0;
	char		szTempPort[MAX_COMPORT_NUM][MAX_PORTNAME_LEN];
	int			iDevID = 0;
	
	m_dwComPortCount = 0;
	memset(szPort, 0, sizeof(szPort));
	memset(m_szPort, 0, sizeof(m_szPort));
	listport.ListPorts(GetComPort, NULL);
	m_dwComPortCount = dwComPortNum;

	memset(szTempPort, 0, sizeof(szTempPort));
	for(i=0; i<m_dwComPortCount; i++)
	{
		iDevID = _wtoi((wchar_t *)(&szPort[i][3]));
		if((iDevID < 256) && (iDevID > 0))
		{
			memcpy(szTempPort[uCount++], szPort[i], MAX_PORTNAME_LEN);
		}
	}
	
	dwComPortNum = uCount;
	m_dwComPortCount = uCount;
	memset(szPort, 0, sizeof(szPort));
	memset(m_szPort, 0, sizeof(m_szPort));
	memcpy(szPort, szTempPort, sizeof(szTempPort));
	memcpy(m_szPort, szTempPort, sizeof(szTempPort));

	for(i=0; i<m_dwComPortCount; i++)
	{
		dwComID[i] = (DWORD)_wtoi((wchar_t *)(&m_szPort[i][3]));
	}
	return m_dwComPortCount;
}

int CUPS::GetCurrentPort(UINT uCommPort)
{
	int i;
	for(i=0; i<(int)m_dwComPortCount; i++)
	{
		if(uCommPort == (UINT) _wtoi((wchar_t *)(&m_szPort[i][3])))
			break;
	}
	if(i >= (int)m_dwComPortCount)
		return -1;
	else
		return i;
}


int CUPS::OpenConnection( UINT uComPort, PortType porttype)
{
	int index = -1;
	char	szName[DEV_NAME_LEN];
	
	
	m_ptDevType = porttype;
	memset(szName, 0, sizeof(szName));

	if(porttype == COM)
	{
		if( (index=GetCurrentPort(uComPort)) == -1)
		{
			if (m_hPortHandle != NULL)
			{
				CloseHandle(m_hPortHandle);
				m_hPortHandle = NULL;
			}
			return -1;
		}

		sprintf_s(szName,DEV_NAME_LEN,"\\\\.\\%s", m_szPort[index]);

		if (NULL != m_hPortHandle)
		{
			CloseHandle(m_hPortHandle);
			m_hPortHandle = NULL;
		}

		m_hPortHandle = CreateFile(szName, 
							GENERIC_READ | GENERIC_WRITE,
							0 , //exclusive
							NULL,
							OPEN_EXISTING,
							FILE_FLAG_OVERLAPPED | FILE_ATTRIBUTE_NORMAL,
							NULL );

		if( m_hPortHandle == INVALID_HANDLE_VALUE )
		{
			if (m_hPortHandle != NULL)
			{
				CloseHandle(m_hPortHandle);
				m_hPortHandle = NULL;
			}
		}

		return 0;
	}

	if(porttype == USB)
	{
		if (NULL != m_hPortHandle)
		{
			CloseHandle(m_hPortHandle);
			m_hPortHandle = NULL;
		}
		m_hPortHandle = OpenUSBDeviceByName(m_UsbDevice.DeviceNamestr); 
		
		if(m_hPortHandle == INVALID_HANDLE_VALUE)
		{
			if (m_hPortHandle != NULL)
			{
				CloseHandle(m_hPortHandle);
				m_hPortHandle = NULL;
			}
		}
		return 0;
	}

	return 0; 
}

BOOL CUPS::SetupConnection( )
{
	if( m_hPortHandle == NULL )
		return FALSE;

    // get any early notifications
	SetCommMask(m_hPortHandle, EV_RXCHAR);

    // setup device buffers
    if ( ! SetupComm( m_hPortHandle , //m_hPortHandle,
        MAXIN_BUFF,
        MAXOUT_BUFF ) )
    {
        DWORD dwErr = GetLastError( );
        return FALSE;
    }

    // purge any information in the buffer
    if ( ! PurgeComm( m_hPortHandle , PURGE_TXABORT | PURGE_RXABORT |
        PURGE_TXCLEAR | PURGE_RXCLEAR ) )
        return FALSE;

    // set DCB
    DCB                dcb;

    // Get current communication port status
    dcb.DCBlength = sizeof( DCB ) ;
    if ( ! GetCommState( m_hPortHandle , & dcb ) )
        return FALSE;

    dcb.BaudRate = CBR_2400;
    dcb.ByteSize = BITESIZE_TRANSMIT_ONCE;
    dcb.StopBits = ONESTOPBIT;
    dcb.Parity = NOPARITY;
    dcb.fDtrControl    = DTR_CONTROL_DISABLE;
    dcb.fRtsControl    = RTS_CONTROL_DISABLE;
    dcb.fOutxCtsFlow = CTS_TRUE_OR_FALSE;
    dcb.fOutxDsrFlow = DSR_TRUE_OR_FALSE;
    dcb.fDsrSensitivity = FALSE;

    dcb.fInX = ZERO;
    dcb.fOutX = ZERO;
	dcb.XonChar	= 17;
	dcb.XoffChar = 19;

    dcb.XonLim = MININPUT_INBUFF;
    dcb.XoffLim = MAXINPUT_INBUFF;
    dcb.fBinary = TRUE;
    dcb.fParity = TRUE;

    if ( ! SetCommState( m_hPortHandle , & dcb ) )
    {
        DWORD err = GetLastError( );
        return FALSE;
    }

   // set TimeOut
    COMMTIMEOUTS    CommTimeOuts;
			
	CommTimeOuts.ReadIntervalTimeout		= MAXDWORD;
	CommTimeOuts.ReadTotalTimeoutMultiplier	= 0;
	CommTimeOuts.ReadTotalTimeoutConstant	= 500;
	CommTimeOuts.WriteTotalTimeoutMultiplier= 2*CBR_9600/CBR_2400;
	CommTimeOuts.WriteTotalTimeoutConstant	= 0;

    if ( ! SetCommTimeouts( m_hPortHandle , & CommTimeOuts ) )
        return FALSE;

    EscapeCommFunction(m_hPortHandle,SETDTR) ;

    return TRUE;
}


void CUPS::ClosePort(PortType porttype)
{
	if( m_hPortHandle != NULL)
	{
		if(porttype == COM)
		{
			SetCommMask( m_hPortHandle , 0 );

			EscapeCommFunction( m_hPortHandle , CLRDTR );
			PurgeComm( m_hPortHandle , PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR ); 
		}
		CloseHandle( m_hPortHandle );
		m_hPortHandle = NULL;
	}
}


	
BOOL CUPS::ReadPort(UINT uLen, char * szUPSData)
{
	char	szData[MAX_UPS_DATA_LENGTH];
	char	szTempData[MAX_UPS_DATA_LENGTH];
	UINT	i = 0;
	int		nRet = 0;
	UINT	uReadLen = 0;
	UINT	nCount = 0;
	DWORD		dwLength, dwErrorFlags;
	OVERLAPPED	osRead = {0};
	COMSTAT		ComStat; 
	BOOL		bRead = FALSE;

	memset(szData, 0, MAX_UPS_DATA_LENGTH);
	memset(szTempData, 0, MAX_UPS_DATA_LENGTH);
	
	if(m_ptDevType == USB)
	{
		if(m_hPortHandle == NULL)
		{
			return FALSE;
		}
		nRet = ReadOneUSBDevice(m_hPortHandle, szData);
		if(nRet == 1)
		{
			strcpy_s(szUPSData,MAX_UPS_DATA_LENGTH, szData);
			return TRUE;
		}
		if(nRet == -3)
		{
			if(m_upsInfo.dwComID > 0)
			{
				ClosePort(USB);
				OpenConnection(m_upsInfo.dwComID - 1,USB);
			}
		}
		return FALSE;
	}

	

	Sleep(READ_DELAY);
	
	memset(&osRead, 0, sizeof(OVERLAPPED));
	osRead.hEvent = CreateEvent(NULL,    // no security
								TRUE,    // explicit reset req
								FALSE,   // initial event reset
								NULL) ;	 // no name
	if(osRead.hEvent == NULL)
	{
		return FALSE;
	}

	ClearCommError(m_hPortHandle, &dwErrorFlags, &ComStat); 
	bRead = ReadFile(m_hPortHandle, szData, uLen, &dwLength, &osRead);

	if(!bRead)
	{
        if (GetLastError() == ERROR_IO_PENDING) 
	    { 
            while(!GetOverlappedResult(m_hPortHandle, &osRead, &dwLength, TRUE )) 
		    { 
				if(GetLastError() == ERROR_IO_INCOMPLETE) 
					continue; 
				else 
				{ 
	                ClearCommError(m_hPortHandle, &dwErrorFlags, &ComStat ); 
					PurgeComm(m_hPortHandle, PURGE_RXCLEAR | PURGE_RXABORT);
					CloseHandle(osRead.hEvent);
					return FALSE;
				}
			}
		}
	}

	else if(uLen != dwLength)
	{
		uReadLen = dwLength;
		nCount = 0;

		while(nCount++ < 10)
		{
			if(!strcmp(szData, "NAK\r") || strstr(szData, "\r"))
			{
				break;
			}

			memset(szTempData, 0, sizeof(szTempData));
			dwLength = 0;

			bRead = ReadFile(m_hPortHandle, szTempData, uLen, &dwLength, &osRead);
			if(!bRead)
			{
				if (GetLastError() == ERROR_IO_PENDING) 
				{ 
					while(!GetOverlappedResult(m_hPortHandle, &osRead, &dwLength, TRUE )) 
					{ 
						if(GetLastError() == ERROR_IO_INCOMPLETE) 
							continue; 
						else 
						{ 
							ClearCommError(m_hPortHandle, &dwErrorFlags, &ComStat ); 
							PurgeComm(m_hPortHandle, PURGE_RXCLEAR | PURGE_RXABORT);
							CloseHandle(osRead.hEvent);
							return FALSE;
						}
					}
				}
			}
			strcat_s(szData, MAX_UPS_DATA_LENGTH, szTempData);
			uReadLen += dwLength;

			if(uReadLen >= uLen)
			{
				break;
			}

			Sleep(READ_DELAY);
		}
	}
			
	PurgeComm(m_hPortHandle,PURGE_RXCLEAR | PURGE_RXABORT);
	CloseHandle(osRead.hEvent);

	if(uLen != 3)	//ReadOk Command has not 0x0d on the end
	{
		for(i=0; i<uLen; i++)
		{
			if(szData[i] == 0x0d)
			{
				szData[i+1] = 0;
				break;
			}
		}
	}
	
	strcpy_s(szUPSData, MAX_UPS_DATA_LENGTH, szData);
	return TRUE;
}

BOOL CUPS::WritePort(const char * szData)
{
	if( m_hPortHandle == NULL )
	return FALSE;
 

	if(m_ptDevType == USB)
	{
		if(WriteOneUSBDevice(m_hPortHandle, szData) == 1)
		{
			return TRUE;
		}
		return FALSE;
	}


    DWORD dwLength , dwRes;
    OVERLAPPED osWrite = {0};
    HANDLE hPort = m_hPortHandle;

    memset( & osWrite , 0 , sizeof( OVERLAPPED ) );
    osWrite.hEvent = CreateEvent( NULL ,    // no security
        TRUE ,    // explicit reset req
        FALSE ,   // initial event reset
        NULL ) ; // no name
    if( osWrite.hEvent == NULL )
    {
        return FALSE;
    }

    PurgeComm( hPort , PURGE_RXCLEAR | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_TXABORT );
    if( ! WriteFile( hPort , szData , (DWORD)strlen(szData), & dwLength , & osWrite ) ) 
    {
        dwRes = WaitForSingleObject( osWrite.hEvent , WAIT_TIME );

        CloseHandle( osWrite.hEvent );
        switch( dwRes )
        {
        case WAIT_OBJECT_0:
            return TRUE;
        default:
            return FALSE;
        }
    }
    CloseHandle( osWrite.hEvent );

    return TRUE;
}
BOOL CUPS::SendInstruction(const char * szOrder)
{ 
	char szCommand[MAX_COMMAND_LENGTH];
	memset(szCommand, 0, MAX_COMMAND_LENGTH);

	strcpy_s(szCommand, MAX_COMMAND_LENGTH, szOrder);
	szCommand[strlen(szOrder)] = 0x0d;
	szCommand[strlen(szOrder) + 1] = NULL;

	return WritePort(szCommand);
}


int LOGONEMSG(char *pszBuff)
{
#if 0
	char szTemp[512] = {0};
	FILE * fp = NULL;
	time_t tNow = time(NULL);
	tm * stTime = localtime(&tNow);
	size_t i = 0;
	char szFileName[MAX_PATH] = {0};
	int nSize = MAX_PATH;
	
	int year = stTime->tm_year + 1900;
	int month = stTime->tm_mon + 1;
	int day = stTime->tm_mday;
	int nHour = stTime->tm_hour;
	int nMin = stTime->tm_min;
	int nSec = stTime->tm_sec;

	sprintf(szTemp, "%04d/%02d/%02d %02d:%02d:%02d   %s\n", 
		year,
		month,
		day,
		nHour,
		nMin,
		nSec,
		pszBuff);

	
	
	if(GetModuleFileName(NULL, szFileName, nSize))
	{
		for(i = strlen(szFileName); i >= 0; i --)
		{
			if(szFileName[i] == '\\')
				break;
		}
		
		szFileName[i+1] = 0;
	}
	strcat(szFileName, "SimpleSDComWithUPS.log");

	
	fp = fopen(szFileName, "at +");
	if (NULL != fp)
	{
		fwrite(szTemp, 1, strlen(szTemp), fp);
		fclose(fp);
	}
#endif
	return 0;
}



BOOL CUPS::UPSReadOK()
{
	char szData[MAX_UPS_DATA_LENGTH];
	memset(szData, 0, sizeof(szData));

	if(ReadPort(3, szData))
	{
		if(strlen(szData) == 3)
		{
			if((szData[0] == 'O') && (szData[1] == 'K'))
			{
				return TRUE;
			}
		}
	}
	
	return FALSE;
}

BOOL CUPS::UPSCmdQ1(char *szData)
{
	UINT i;
	char szTemp[512];
	

	//  [04/21/2010 orangexie] set mutex for operation usb or port begin
	SECURITY_ATTRIBUTES secuAttr;
	SECURITY_DESCRIPTOR	sd;	
	HANDLE hMutex = NULL;
	BOOL bWinNT = FALSE;
	int nRetry = 0;
	DWORD dwWaitRet = 0;
	HANDLE hMutex_create = NULL;
	OSVERSIONINFO osvi;
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx (&osvi);
	
	for (i = 0; i < 10; i ++)
	{
		if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
		{
			secuAttr.nLength = sizeof( SECURITY_ATTRIBUTES );
			secuAttr.bInheritHandle = FALSE;
			InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION);
			SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
			secuAttr.lpSecurityDescriptor = &sd;
			hMutex = CreateMutex(&secuAttr, FALSE, MUTEX_FOR_OPERATION_UPS);
		}
		else
		{
			hMutex = CreateMutex(NULL, FALSE, MUTEX_FOR_OPERATION_UPS);
		}
		if (GetLastError() == ERROR_ALREADY_EXISTS)
		{
			if (hMutex)
			{
				CloseHandle(hMutex);
				hMutex = NULL;
			}
		}
		else
		{
			break;
		}
		Sleep(100);
	}

	//  [04/21/2010 orangexie] set mutex for operation usb or port end



	for(i=0; i<MAX_PORT_FAIL_LOOP; i++)
	{
		if(!SendInstruction("Q1"))
		{
			memset(szTemp, 0x00, sizeof(szTemp));
			sprintf(szTemp, "The %dst try to send command \"Q1\" to UPS fail.", i + 1);
			LOGONEMSG(szTemp);
			continue;
		}
		memset(szTemp, 0x00, sizeof(szTemp));
		sprintf(szTemp, "The %dst try to send command \"Q1\" to UPS success.", i + 1);
		LOGONEMSG(szTemp);
	
		if(ReadPort(47, szData))
		{
			if(strlen(szData) == 47)
			{
				memset(szTemp, 0x00, sizeof(szTemp));
				strcpy(szTemp, "Get command \"Q1\" respond from UPS success.");
				LOGONEMSG(szTemp);
				ReleaseMutex(hMutex);
				CloseHandle(hMutex);
				hMutex = NULL;
				return TRUE;
			}
			else
			{
				memset(szTemp, 0x00, sizeof(szTemp));
				strcpy(szTemp, "Get command \"Q1\" respond from UPS, but the packet construction is wrong.");
				LOGONEMSG(szTemp);
			}
		}
		else
		{
			memset(szTemp, 0x00, sizeof(szTemp));
			strcpy(szTemp, "Try to get command \"Q1\" respond from UPS fail.");
			LOGONEMSG(szTemp);
			Sleep(200);
		}
	}
	ReleaseMutex(hMutex);
	CloseHandle(hMutex);
	hMutex = NULL;
	return FALSE;
}

BOOL CUPS::UPSCmdSN(UINT uDown)
{
	int 	i = 0;
	char 	szCommand[15];
	char szTemp[512];
	char 	szCommand2[31];

	if (m_nOmronStatus == UPS_STATUS_ONLY_SD_OS)
	{
		return TRUE;
	}


	//  [04/21/2010 orangexie] set mutex for operation usb or port begin
	SECURITY_ATTRIBUTES secuAttr;
	SECURITY_DESCRIPTOR	sd;	
	HANDLE hMutex = NULL;
	BOOL bWinNT = FALSE;
	int nRetry = 0;
	DWORD dwWaitRet = 0;
	HANDLE hMutex_create = NULL;
	OSVERSIONINFO osvi;
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx (&osvi);	
	
	for (i = 0; i < 10; i ++)
	{
		if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
		{
			secuAttr.nLength = sizeof( SECURITY_ATTRIBUTES );
			secuAttr.bInheritHandle = FALSE;
			InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION);
			SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
			secuAttr.lpSecurityDescriptor = &sd;
			hMutex = CreateMutex(&secuAttr, FALSE, MUTEX_FOR_OPERATION_UPS);
		}
		else
		{
			hMutex = CreateMutex(NULL, FALSE, MUTEX_FOR_OPERATION_UPS);
		}
		if (GetLastError() == ERROR_ALREADY_EXISTS)
		{
			if (hMutex)
			{
				CloseHandle(hMutex);
				hMutex = NULL;
			}
		}
		else
		{
			break;
		}
		Sleep(100);
	}
	//  [04/21/2010 orangexie] set mutex for operation usb or port end


	memset(szCommand, 0, sizeof(szCommand));
//colin modify for shutdown time 600 + extern command time 600 @2011/01/17
	if(uDown > 1200)
	{
		uDown = 1200;
	}
//end colin modify
	if(uDown < 12)
	{
		uDown = 12;	
	}
	
	if(uDown >= 60)
	{
		sprintf_s(szCommand, "S%02u", uDown/60);
	}
	else
	{
		sprintf_s(szCommand, "S.%01u", uDown/6);
	}
	if (m_nOmronStatus == UPS_STATUS_SD_OS_RESTART_LATER)
	{
		memset(szCommand2, 0, sizeof(szCommand2));
//colin added for ups RestartTime
		m_nRestartTime = m_nRestartTime - (uDown/60);
		if (m_nRestartTime < 1) m_nRestartTime = 1;
//end colin added
		sprintf_s(szCommand2, "%sR%05u", szCommand, m_nRestartTime);
	}
	else if ( m_nOmronStatus != 0 )
	{
		memset(szCommand2, 0, sizeof(szCommand2));

		//imachine Modified 2010.07.14
		//strcpy(szCommand2, szCommand);
		if(uDown >= 60)
			sprintf_s(szCommand2, "Sf%02u", uDown/60);
		else
			sprintf_s(szCommand2, "Sf.%01u", uDown/6);
	}
	else
	{
		memset(szCommand2, 0, sizeof(szCommand2));
		strcpy(szCommand2, szCommand);
	}

	for(i=0; i<MAX_PORT_FAIL_LOOP; i++)
	{
		if(SendInstruction(szCommand2))
		{
			memset(szTemp, 0x00, sizeof(szTemp));
			sprintf(szTemp, "The %dst try to send command \"%s\" to UPS success.", i + 1, szCommand);
			LOGONEMSG(szTemp);
			if(UPSReadOK())
			{
				memset(szTemp, 0x00, sizeof(szTemp));
				sprintf(szTemp, "Get command \"%s\" respond from UPS success.", szCommand);
				LOGONEMSG(szTemp);
				ReleaseMutex(hMutex);
				CloseHandle(hMutex);
				hMutex = NULL;
				return TRUE;
			}
			else
			{
				memset(szTemp, 0x00, sizeof(szTemp));
				sprintf(szTemp, "Get command \"%s\" respond from UPS fail.", szCommand);
				LOGONEMSG(szTemp);
			}
		}
		else
		{
			Sleep(200);
			memset(szTemp, 0x00, sizeof(szTemp));
			sprintf(szTemp, "The %dst try to send command \"%s\" to UPS fail.", i + 1, szCommand);
			LOGONEMSG(szTemp);
		}
	}
	ReleaseMutex(hMutex);
	CloseHandle(hMutex);
	hMutex = NULL;
	return FALSE;
}

BOOL CUPS::SearchComUPS(UINT uComPort)
{
	char szData[MAX_UPS_DATA_LENGTH];
	memset(szData, 0, MAX_UPS_DATA_LENGTH);

	ClosePort(COM);
	OpenConnection(uComPort, COM);

	if(m_hPortHandle != NULL)
	{
		if(SetupConnection())
		{
			if(UPSCmdQ1(szData))
			{
				ClosePort(COM);
				return TRUE;
			}
		}
		else
		{
			CloseHandle(m_hPortHandle);
			m_hPortHandle = NULL;
		}

		ClosePort(COM);
	}

	return FALSE;
}

BOOL CUPS::ConnectUPS(UINT uComPort, PortType porttype)
{
	char szData[MAX_UPS_DATA_LENGTH];
	memset(szData, 0, MAX_UPS_DATA_LENGTH);

	ClosePort(porttype);
	OpenConnection(uComPort, porttype);
	if(m_hPortHandle != NULL)
	{
		if(porttype == COM)
		{
			if(!SetupConnection())
			{
				CloseHandle(m_hPortHandle);
				m_hPortHandle = NULL;
				ClosePort(COM);
				return FALSE;
			}
		}

		if(UPSCmdQ1(szData))
		{
			return TRUE;
		}
		
		ClosePort(porttype);
	}

	return FALSE;
}



BOOL CUPS::PollingUPS()
{
	char		szData[MAX_UPS_DATA_LENGTH] = { 0 };

	memset(szData, 0, sizeof(szData));


	if(!UPSCmdQ1(szData))
	{
		return FALSE;
	}

	m_dwUPSStatus = 0;
	if(szData[38] == '1')
	{
		m_dwUPSStatus |= UPS_STATUS_ACFAIL;
	}

	if(szData[39] == '1')
	{
		m_dwUPSStatus |= UPS_STATUS_BATTLOW;
	}

	if(szData[44] == '1')
	{
		m_dwUPSStatus |= UPS_STATUS_SHUTDOWN;
	}

	if (m_nOmronStatus == UPS_STATUS_SD_OS_UPS)
	{
		m_dwUPSStatus = 0;
		m_dwUPSStatus |= UPS_STATUS_SD_OS_UPS;
	}
	if (m_nOmronStatus == UPS_STATUS_SD_OS_RESTART_LATER)
	{
		m_dwUPSStatus = 0;
		m_dwUPSStatus |= UPS_STATUS_SD_OS_RESTART_LATER;
	}
	if (m_nOmronStatus == UPS_STATUS_ONLY_SD_OS)
	{
		m_dwUPSStatus = 0;
		m_dwUPSStatus |= UPS_STATUS_ONLY_SD_OS;
	}

	m_upsInfo.dwUPSStatus = m_dwUPSStatus;

	return TRUE;
}

BOOL CUPS::OpenOneComDevice(int *comId ,BOOL bAuto)
{
	DWORD arrComId[MAX_COMPORT_NUM];
	memset(arrComId, 0,MAX_COMPORT_NUM);
	m_dwComPortCount = 0;
   
	m_dwComPortCount = ListComPorts(arrComId);

	if (m_dwComPortCount > 0)
	{
		if (bAuto)
		{
			for (int i=0;i < (int)m_dwComPortCount; i++)
			{
				if( SearchComUPS(arrComId[i]))
				{
					*comId = arrComId[i];
					return TRUE;
				}
			}
		}
		else
		{
			if( SearchComUPS(*comId))
			{
				return TRUE;
			}
		}
	}

	return FALSE;
}

